class UsersGetAllOrders(object):
    def __init__(self, key, user_id):
        self.key = key
        self.user_id = user_id

    # 业务密钥
    key = ""
    # 用户id
    user_id = ""
    # 产品类型
    product_type = ""



class DynamicReplaceWhiteIp(object):
    def __init__(self, key, trade_no):
        self.key = key
        self.trade_no = trade_no

    # 业务秘钥
    key = ""
    # 业务编号
    trade_no = ""
    # ip列表
    old_ip = ""
    # 已存在,需要被替换的白名单IP
    new_ip = ""
    # 是否重置已存在的白名单IP
    reset = ""


import hashlib


# 计算签名
def get_params(kv: {}, appkey: str) -> str:
    # 清理参数
    cleanParams = clean_params(kv)
    value = format_url_map(cleanParams, 0, 0)
    value += "&key=" + appkey
    sign = hashlib.md5(value.encode(encoding='UTF-8')).hexdigest()
    cleanParams["sign"] = sign
    params = format_url_map(cleanParams, 0, 0)
    return "?" + params


# 在字典中清理不用的参数
def clean_params(kv: {}) -> {}:
    para_map = {}
    for i in kv:
        value = kv[i]
        if value == "":
            continue
        if i == "key":
            continue
        para_map[i] = value
    return para_map


# 方法用途: 对所有传入参数按照字段名的 ASCII 码从小到大排序（字典序），并且生成url参数串
def format_url_map(para_map: {}, url_encode: bool, key_toLower: bool) -> str:
    buff = ""
    for i in sorted(para_map):
        if i == "key":
            continue
        buff += "&" + i + "=" + para_map[i]
    buff = buff[1:]
    return buff

# def main():
#     paraMap = {}
#     paraMap["trade_no"] = "1135123858735679"
#     paraMap["num"] = "5"
#     paraMap["pt"] = "2"
#     paraMap["city_name"] = "1"
#     paraMap["result_type"] = "json"
#     paraMap["key"] = "0794d170bdc14b3f835a2202bb21cbcd"
#     params = get_params(paraMap, paraMap["key"])
#     print(params)
#
#
# if __name__ == "__main__":
#     main()



from .common.DynamicGetIps import DynamicGetIps
from .common.DynamicCheck import DynamicCheck
from .common.DynamicSetWhiteIp import DynamicSetWhiteIp
from .common.DynamicGetWhiteIp import DynamicGetWhiteIp
from .common.DynamicRemain import DynamicRemain
from .common.DynamicBalance import DynamicBalance
from .common.DynamicReplaceWhiteIp import DynamicReplaceWhiteIp
from .common.UsersGetBalance import UsersGetBalance
from .common.AloneGetIps import AloneGetIps
from .common.AloneSetWhiteIp import AloneSetWhiteIp
from .common.AloneGetWhiteIp import AloneGetWhiteIp
from .common.AloneReplaceWhiteIp import AloneReplaceWhiteIp
from .common.UsersGetAllOrders import UsersGetAllOrders
from .enums.URL import URL
from .ext.StrKit import get_params
import urllib.request
from urllib.parse import unquote


# 动态代理 --提取动态代理
def dynamic_get_ips(getips: DynamicGetIps) -> str:
    dic = getips.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.DYNAMIC_GETIPS.value + params)
    result = unquote(request.read(), "utf-8")
    return result


# 动态代理 -- 校验IP可用性
def dynamic_check(check: DynamicCheck) -> str:
    dic = check.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.DYNAMIC_CHECK.value + params)
    result = unquote(request.read(), "utf-8")
    return result


# 动态代理 -- 设置代理IP白名单
def dynamic_set_white_ip(ip: DynamicSetWhiteIp) -> str:
    dic = ip.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.DYNAMIC_SETWHITEIP.value + params)
    result = unquote(request.read(), "utf-8")
    return result

# 动态代理 -- 替换IP白名单
def dynamic_replace_white_ip(ip: DynamicReplaceWhiteIp) -> str:
    dic = ip.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.DYNAMIC_REPLACEWHITEIP.value + params)
    result = unquote(request.read(), "utf-8")
    return result


# 动态代理 -- 获取IP白名单
def dynamic_get_white_ip(ip: DynamicGetWhiteIp) -> str:
    dic = ip.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.DYNAMIC_GETWHITEIP.value + params)
    result = unquote(request.read(), "utf-8")
    return result


# 动态代理 -- 获取代理剩余可用时长
def dynamic_remain(remain: DynamicRemain) -> str:
    dic = remain.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.DYNAMIC_REMAIN.value + params)
    result = unquote(request.read(), "utf-8")
    return result


# 动态代理 -- 获取剩余可提取IP数量
def dynamic_balance(balance: DynamicBalance) -> str:
    dic = balance.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.DYNAMIC_BALANCE.value + params)
    result = unquote(request.read(), "utf-8")
    return result

def users_get_allOrders(allorders: UsersGetAllOrders) -> str:
    dic = allorders.__dict__
    appkey = dic.get("key")
    params = get_params(dic,appkey)
    request = urllib.request.urlopen(URL.USERS_GETALLORDERS.value + params)
    result = unquote(request.read(), "utf-8")
    return result

# 获取账户余额
def users_get_balance(balance: UsersGetBalance) -> str:
    dic = balance.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.USERS_GETBALANCE.value + params)
    result = unquote(request.read(), "utf-8")
    return result


# 独享代理 -- 获取代理详情
def alone_get_ips(ips: AloneGetIps) -> str:
    dic = ips.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.ALONE_GETIPS.value + params)
    result = unquote(request.read(), "utf-8")
    return result


# 独享代理 -- 设置代理IP白名单
def alone_set_white_ip(ip: AloneSetWhiteIp) -> str:
    dic = ip.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.ALONE_SETWHITEIP.value + params)
    result = unquote(request.read(), "utf-8")
    return result

# 独享代理 -- 替换IP白名单
def alone_replace_white_ip(ip: AloneReplaceWhiteIp) -> str:
    dic = ip.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.ALONE_REPLACEWHITEIP.value + params)
    result = unquote(request.read(), "utf-8")
    return result



# 独享代理 -- 获取代理IP白名单
def alone_get_white_ip(ip: AloneGetWhiteIp) -> str:
    dic = ip.__dict__
    appkey = dic.get("key")
    params = get_params(dic, appkey)
    request = urllib.request.urlopen(URL.ALONE_GETWHITEIP.value + params)
    result = unquote(request.read(), "utf-8")
    return result




class AloneGetIps(object):
    def __init__(self, key, trade_no):
        self.key = key
        self.trade_no = trade_no

    # 密钥
    key = ""
    # 业务编号
    trade_no = ""
    # SOCK代理端口
    sock_port = ""
    # IP可用时间[动态型独有]
    ip_remain = ""
    # 地区名称
    city_name = ""
    # 邮政编码
    city_code = ""
    # 业务到期时间
    order_endtime = ""



from enum import Enum, unique


@unique
class URL(Enum):
    # 主站地址
    DOMAIN = "https://v1.api.juliangip.com"
    # DOMAIN = "http://192.168.10.52:8087"
    # 获取账户余额
    USERS_GETBALANCE = DOMAIN + "/users/getbalance"
    # 获取账户下对应类型的所有正常状态订单
    USERS_GETALLORDERS = DOMAIN + "/users/getAllOrders"
    # 动态代理 - - 提取动态代理
    DYNAMIC_GETIPS = DOMAIN + "/dynamic/getips"
    # 动态代理 - - 校验IP可用性
    DYNAMIC_CHECK = DOMAIN + "/dynamic/check"
    # 动态代理 - - 设置代理IP白名单
    DYNAMIC_SETWHITEIP = DOMAIN + "/dynamic/setwhiteip"
    # 动态代理 - - 替换IP白名单
    DYNAMIC_REPLACEWHITEIP = DOMAIN + "/dynamic/replaceWhiteIp"
    # 动态代理 - - 获取IP白名单
    DYNAMIC_GETWHITEIP = DOMAIN + "/dynamic/getwhiteip"
    # 动态代理 - - 获取代理剩余可用时长
    DYNAMIC_REMAIN = DOMAIN + "/dynamic/remain"
    # 动态代理 - - 获取剩余可用时长
    DYNAMIC_BALANCE = DOMAIN + "/dynamic/balance"
    # 独享代理 - - 获取代理详情
    ALONE_GETIPS = DOMAIN + "/alone/getips"
    # 独享代理 - - 设置代理IP白名单
    ALONE_SETWHITEIP = DOMAIN + "/alone/setwhiteip"
    # 独享代理 - - 获取代理IP白名单
    ALONE_GETWHITEIP = DOMAIN + "/alone/getwhiteip"
    # 独享代理 -- 替换IP白名单
    ALONE_REPLACEWHITEIP = DOMAIN + "/alone/replaceWhiteIp"



class DynamicGetIps(object):
    def __init__(self, key, trade_no, num):
        self.key = key
        self.trade_no = trade_no
        self.num = num


    # 业务秘钥
    key = ""
    # 业务编号
    trade_no = ""
    # 提取数量
    num = ""
    # 代理类型
    pt = ""
    # 返回类型
    result_type = ""
    # 结果分隔符
    split = ""
    # 地区名称
    city_name = ""
    # 邮政编码
    city_code = ""
    # 剩余可用时长
    ip_remain = ""
    # 筛选地区
    area = ""
    # 排除地区
    no_area = ""
    # 筛选ip段
    ip_seg = ""
    # 排除IP段
    no_ip_seg = ""
    # 运营商筛选
    isp = ""
    # IP去重
    filter = ""
